/******************************************************************************
 *   irq.h:  Interrupt related Header file for Philips LPC214x Family 
 *   Microprocessors
 *
 *   Copyright(C) 2006, Philips Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2005.10.01  ver 1.00    Prelimnary version, first Release
 *
 *   21. Apr. 2006 - modified for gnu/gcc by Martin Thomas
 * 
 *   17. Mar. 2007 let: - Included enable/disable/restore functions from WinARM examples.
 *                      - Disabled nested interrupts
 *   22. Apr. 2007 let: - Added GCC/Thumb IRQ workaround
 ******************************************************************************/
#ifndef IRQ_H 
#define IRQ_H

#include <stdint.h>

#if THUMB_SUPPORT
#define __irq __attribute__((naked))
#define ISR_ENTER() asm volatile(" sub   lr, lr,#4\n" \
                                 " stmfd sp!,{r0-r12,lr}\n" \
                                 " mrs   r1, spsr\n" \
                                 " stmfd sp!,{r1}")
#define ISR_EXIT() VICVectAddr = 0; asm volatile(" ldmfd sp!,{r1}\n" \
                                 " msr   spsr_c,r1\n" \
                                 " ldmfd sp!,{r0-r12,pc}^")
#else
#define __irq __attribute__ ((interrupt("IRQ")))
#define ISR_ENTER()
#define ISR_EXIT() VICVectAddr = 0
#endif

//#define __fiq __attribute__ ((interrupt("FIQ")))

#define	WDT_INT			0
#define SWI_INT			1
#define ARM_CORE0_INT	2
#define	ARM_CORE1_INT	3
#define	TIMER0_INT		4
#define TIMER1_INT		5
#define UART0_INT		6
#define	UART1_INT		7
#define	PWM0_INT		8
#define I2C0_INT		9
#define SPI0_INT		10
#define	SPI1_INT		11
#define	PLL_INT			12
#define RTC_INT			13
#define EINT0_INT		14
#define EINT1_INT		15
#define EINT2_INT		16
#define EINT3_INT		17
#define	ADC0_INT		18
#define I2C1_INT		19
#define BOD_INT			20
#define ADC1_INT		21
#define USB_INT			22

#define IRQ_SLOT_EN		0x20  // bit 5 in Vector control register
#define VIC_SIZE		16

#define VECT_ADDR_INDEX	0x100
#define VECT_CNTL_INDEX 0x200

#ifdef __cplusplus
	extern "C" {
#endif

void irq_init(void);

uint32_t irq_install(uint32_t IntNumber, void *HandlerAddr);
uint32_t irq_uninstall(uint32_t IntNumber);
uint32_t irq_enableIRQ(void);
uint32_t irq_disableIRQ(void);
uint32_t irq_restoreIRQ(uint32_t oldCPSR);

#ifdef __cplusplus
	}
#endif

#endif /* end IRQ_H */
